# Copyright (c) 2025 YowFung (yowfung@outlook.com)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from huggingface_hub import HfApi, snapshot_download
from retrying import retry

# Available mirror sites
MIRROR_SITES = [
    "https://hf-mirror.com",
    "https://huggingface.co",
    "https://huggingface.co/api",
]


def get_working_mirror():
    """Test and return a working mirror site"""
    api = HfApi()
    for mirror in MIRROR_SITES:
        try:
            os.environ["HF_ENDPOINT"] = mirror
            # Try accessing a public model to test connection
            api.model_info("gpt2")
            return mirror
        except Exception as e:
            continue
    raise RuntimeError("All mirror sites are unavailable.")


@retry(stop_max_attempt_number=3, wait_fixed=2000)
def download_pretrained_model(
    model_name: str = "SparkAudio/Spark-TTS-0.5B",
    local_dir: str = "pretrained_models/Spark-TTS-0.5B",
    token: str = None,
):
    """Download the pretrained model.

    Args:
        model_name (str): The name of the model to download.
        local_dir (str): The local directory to save the model.
        token (str, optional): Hugging Face token for authentication.
    """
    # Get a working mirror
    mirror = get_working_mirror()
    os.environ["HF_ENDPOINT"] = mirror
    os.environ["HF_HUB_ENABLE_HF_TRANSFER"] = "1"  # Enable acceleration

    # If no token is provided, try to get it from the environment variable
    if token is None:
        token = os.environ.get("HUGGING_FACE_HUB_TOKEN")

    # Ensure local_dir is an absolute path
    local_dir = os.path.abspath(local_dir)

    snapshot_download(
        repo_id=model_name,
        local_dir=local_dir,
        token=token,
    )


if __name__ == "__main__":
    download_pretrained_model()



import os
import time

import numpy as np
import pytest
import torch

from spark_tts_lib.SparkTTS import SparkTTS
from spark_tts_lib.utils.download import download_pretrained_model


def run_inference_test(device: torch.device):
    """General inference test function

    Args:
        device (torch.device): Running device (CPU/GPU)
    """
    model_dir = "pretrained_models/Spark-TTS-0.5B"
    if not os.path.exists(model_dir):
        print("⌛️ Downloading pretrained model...")
        download_pretrained_model(local_dir=model_dir)
        print("☑️ Downloaded pretrained model.")

    print("⌛️ Loading model...")
    start_at = time.time()
    model = SparkTTS(model_dir, device)
    elapsed_time = time.time() - start_at
    print(f"☑️ Model loaded in {elapsed_time:.2f} seconds.")

    text = "生活就像海洋，只有意志坚强的人才能到达彼岸。"
    prompt_speech_path = "tests/prompt_audio.wav"
    prompt_text = "吃燕窝就选燕之屋，本节目由26年专注高品质燕窝的燕之屋冠名播出。豆奶牛奶换着喝，营养更均衡，本节目由豆本豆豆奶特约播出。"

    print("⌛️ Inferring...")
    start_at = time.time()
    wav = model.inference(text, prompt_speech_path, prompt_text)

    elapsed_time = time.time() - start_at
    print(f"☑️ Inference completed in {elapsed_time:.2f} seconds.")

    assert isinstance(wav, np.ndarray)
    assert wav.ndim == 1
    assert len(wav) > 1000

    print("✅ Inference test passed.")


@pytest.mark.order(1)
def test_inference_use_cpu():
    """Test CPU inference"""
    run_inference_test(torch.device("cpu"))


@pytest.mark.order(2)
def test_inference_use_cuda():
    """Test CUDA inference"""
    if not torch.cuda.is_available():
        pytest.skip("CUDA is not available")
    run_inference_test(torch.device("cuda"))
